#!/bin/bash
set -evx
#
# This script computes the ocean heat content from the NEMO output
#
# Written by Hui Du (based on "/gpfs/projects/ecm86/common/cdftools/scripts/cdfheatc.sh" on BSC)
#
# Created:   October 1, 2010
#            Hui Du (based on "/gpfs/projects/ecm86/common/cdftools/scripts/cdfheatc.sh" on BSC)
# Modified:  October 22, 2010
#            F. Doblas-Reyes
#
########################
### General Settings ###
########################
EXPID=b013  # experiment ID
#YEAR_LST=( 19601101 19651101 19701101 19751101 19801101 19851101 19901101 19951101 20001101 20051101 )
YEAR_LST=( 19951101 )
MEMBER_LST=( fc0 fc1 fc2 fc3 fc4 )
#MEMBER_LST=( fc0 )
DATADIR=/scratch
#
# The data directory should follow the structure ${DATADIR}/${EXPID}/${year}/${member}/
# where ${DATADIR} is the directory where the model output is available from
# $EXPID is the experiment ID
# $year is the starting date of the simulation
# $member is the ensemble member
#
#TMPDIR=/cfu/scratch/${USER}/cdfheatc_$$
TMPDIR=/tmp/${USER}/heatc/$$
SAVEDIR=/cfu/scratch/${USER}/heatc/${EXPID}
#
# Create setup
#
mkdir -p ${SAVEDIR}
rm -rf $TMPDIR
mkdir -p $TMPDIR
cd $TMPDIR
#
# Modules and libraries
#
cdo=`which cdo`
ncrename=`which ncrename`
ncdump=`which ncdump`
ncatted=`which ncatted`
#
# Check the existence of these tools
#
#if tool=$cdo      && test ! -f $tool || \
#   tool=$ncrename && test ! -f $tool || \
#   tool=$ncatted  && test ! -f $tool || \
#   echo "$tool does not exist!!!"
#   exit 1
#fi
cdftools=/opt/cdftools/bin
con_files=/cfu/pub/scripts/CDFTOOLS/constant_files
#
# Description of cdfheatc
#
#  Usage : cdfheatc  gridTfile  [imin imax jmin jmax kmin kmax]
#  Computes the heat content in the specified area (Joules)
#  imin imax jmin jmax kmin kmax can be given as options
#     if imin = 0 then ALL i are taken
#     if jmin = 0 then ALL j are taken
#     if kmin = 0 then ALL k are taken
#  PARTIAL CELLS VERSION
#  Files mesh_hgr.nc, mesh_zgr.nc, mask.nc
#  must be in the current directory
#  Output on standard output
#
# Initialization
#
imin=0
imax=0
jmin=0
jmax=0
kmin=1
kmax=24  # 24 for about 657m and 20 for 315m
grid=grid_T
exec=cdfheatc
#
cp $cdftools/$exec .
#cp ${HOME}/${exec} .
#cp /home/huidu/${exec} .
cp ${con_files}/depth.txt .
ln -sf ${con_files}/mask.nc .
ln -sf ${con_files}/mesh_hgr.nc .
ln -sf ${con_files}/mesh_zgr.nc .
#
para="$imin $imax $jmin $jmax $kmin $kmax"
if [[ ${kmin} -eq 1 ]] ; then
   kmin=0
else
   kmin=`cat depth.txt |head -${kmin} |tail -1 | awk '{print $2}' | awk '{printf "%.0f",$1}'`;
fi
kmax=`cat depth.txt |head -${kmax} |tail -1 | awk '{print $2}' | awk '{printf "%.0f",$1}'`;
#
prefix=ohc
suffix=${kmin}-${kmax}_global
#
# Loop over start dates and members
#
for year in ${YEAR_LST[@]}; do
    echo "******************** Start $year **********************"
    date
    for member in ${MEMBER_LST[@]}; do
        echo "******************** Start $member **********************"
        echo
#
# File retrieval and variable selection
#
#       scp $USER@forseti:${DATADIR}/$EXPID/$year/$member/MMO_${EXPID}_${year}_${member}_*.tar .
        source_dir=${DATADIR}/$EXPID/$year/$member
        cd $source_dir; files=`ls MMO*${EXPID}_${year}_${member}_*.tar`;cd $TMPDIR
        for file in $files; do
            file1=`tar tf $source_dir/$file | grep ${grid}`
            tar xvf $source_dir/$file ${file1}
            gunzip ${file1}
            file2=`echo $file1 |cut -c1-36`
            rm -f ${file2}.tmp
            ncks -v votemper,nav_lon,nav_lat,deptht,time_counter ${file2} -o ${file2}.tmp
            mv ${file2}.tmp ${file2}
        done
        merged=${EXPID}_${year}_${member}_ORCA1
        rm -f ${merged}
        $cdo mergetime ORCA1_MM_*_${grid}.nc ${merged}.nc
        chmod 777 ${EXPID}*.nc
#
# Calculation of the ohc
#
        output=${prefix}_${merged}_${suffix}
        cp ${con_files}/heatc.nc toto.nc # heatc.nc is a file that contains a template for the ohc NetCDF output
        ntime=`$cdo ntime ${merged}.nc`;
        rm -f output.txt
        for t in $(seq 1 1 $ntime); do
            rm -f tmp.nc
            $cdo seltimestep,$t ${merged}.nc tmp.nc
            ./$exec tmp.nc $para > tmp.log
            thc=`cat tmp.log | grep "Total Heat content :" | awk '{print $5}'`;
            uhc=`cat tmp.log | grep "Total Heat content/volume" | awk '{print $5}'`;
            echo $t $thc $uhc >> ${output}.txt
            ncap2 -h -O -s "thc()=$thc;uhc()=$uhc" toto.nc toto.nc
            if [[ $t = 1 ]] ; then
               cp toto.nc ${output}.nc
            else
               ncrcat -h -O ${output}.nc toto.nc ${output}.nc
            fi
        done
        ncks -h -O -v time ${merged}.nc -o time.nc
        ncks -h -A time.nc ${output}.nc
#
# Modify headers
#
        thc_min=`cat ${output}.txt | awk '{print $2}' | sort -n | head -1`;
        thc_max=`cat ${output}.txt | awk '{print $2}' | sort -n | tail -n 1`;
        $ncatted -h -a valid_min,thc,m,f,$thc_min ${output}.nc
        $ncatted -h -a valid_max,thc,m,f,$thc_max ${output}.nc
#
        uhc_min=`cat ${output}.txt | awk '{print $3}' | sort -n | head -1`;
        uhc_max=`cat ${output}.txt | awk '{print $3}' | sort -n | tail -n 1`;
        $ncatted -h -a valid_min,uhc,m,f,$uhc_min ${output}.nc
        $ncatted -h -a valid_max,uhc,m,f,$uhc_max ${output}.nc
#  
        units=`$ncdump -c ${merged}.nc | grep "time:units" | cut -d '"' -f2`;
        $ncatted -h -a units,time,m,c,"$units" ${output}.nc
        $ncatted -h -a time,time,d,, ${output}.nc
#
        rm -f MMO_${EXPID}_${year}_${member}_*
        rm -f ORCA1_MM_*
        rm -f ${merged}.nc
#
        echo "******************** END $member  *********************"
    done
    echo "******************** END $year  *********************"
done
#
mv ohc*.nc ${SAVEDIR}; mv ohc*.txt ${SAVEDIR}
cd ..
rm -rf ${TMPDIR}
date
exit 0
